'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys;

function _load_keys() {
  return _keys = _interopRequireDefault(require('babel-runtime/core-js/object/keys'));
}

exports.getRcConf = getRcConf;
exports.getRcArgs = getRcArgs;
exports.clearRcCache = clearRcCache;

var _parse;

function _load_parse() {
  return _parse = _interopRequireDefault(require('./lockfile/parse.js'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const rc = require('rc');

// Keys that will get resolved relative to the path of the rc file they belong to


const PATH_KEYS = ['cache-folder', 'global-folder', 'modules-folder'];

let rcConfCache;
let rcArgsCache;

const buildRcConf = () => rc('yarn', {}, [], fileText => {
  const values = (0, (_parse || _load_parse()).default)(fileText, 'yarnrc');
  const keys = (0, (_keys || _load_keys()).default)(values);

  // Unfortunately, the "rc" module we use doesn't tell us the file path :(
  // cf https://github.com/dominictarr/rc/issues/61

  for (const key of keys) {
    for (const pathKey of PATH_KEYS) {
      if (key.replace(/^(--)?([^.]+\.)+/, '') === pathKey) {
        // values[key] = resolve(dirname(filePath), values[key]);
        if (!values[key].startsWith('/')) {
          delete values[keys];
        }
      }
    }
  }

  return values;
});

function getRcConf() {
  if (!rcConfCache) {
    rcConfCache = buildRcConf();
  }

  return rcConfCache;
}

const buildRcArgs = () => (0, (_keys || _load_keys()).default)(getRcConf()).reduce((argLists, key) => {
  const miniparse = key.match(/^--(?:([^.]+)\.)?(.*)$/);

  if (!miniparse) {
    return argLists;
  }

  const namespace = miniparse[1] || '*';
  const arg = miniparse[2];
  const value = getRcConf()[key];

  if (!argLists[namespace]) {
    argLists[namespace] = [];
  }

  if (typeof value === 'string') {
    argLists[namespace] = argLists[namespace].concat([`--${arg}`, value]);
  } else if (value === true) {
    argLists[namespace] = argLists[namespace].concat([`--${arg}`]);
  } else if (value === false) {
    argLists[namespace] = argLists[namespace].concat([`--no-${arg}`]);
  }

  return argLists;
}, {});

function getRcArgs(command) {
  if (!rcArgsCache) {
    rcArgsCache = buildRcArgs();
  }

  let result = rcArgsCache['*'] || [];

  if (command !== '*') {
    result = result.concat(rcArgsCache[command] || []);
  }

  return result;
}

function clearRcCache() {
  rcConfCache = null;
  rcArgsCache = null;
}